"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/* tslint:disable:no-unused-expression object-literal-sort-keys */
var chai = require("chai");
var chaiAsPromised = require("chai-as-promised");
var sinon = require("sinon");
var sinonChai = require("sinon-chai");
var request_1 = require("./common/request");
var interaction_1 = require("./dsl/interaction");
var mockService_1 = require("./dsl/mockService");
var pact_web_1 = require("./pact-web");
var expect = chai.expect;
chai.use(sinonChai);
chai.use(chaiAsPromised);
describe("PactWeb", function () {
    var fullOpts = {
        consumer: "A",
        cors: false,
        host: "127.0.0.1",
        logLevel: "info",
        pactfileWriteMode: "overwrite",
        port: 1234,
        provider: "B",
        spec: 2,
        ssl: false,
    };
    var sandbox = sinon.sandbox.create({
        injectInto: null,
        properties: ["spy", "stub", "mock"],
        useFakeServer: false,
        useFakeTimers: false,
    });
    afterEach(function () {
        sandbox.restore();
    });
    describe("#constructor", function () {
        it("throws Error when consumer not provided", function () {
            expect(function () {
                new pact_web_1.PactWeb({ consumer: "", provider: "provider" });
            }).not.to.throw(Error, "You must specify a Consumer for this pact.");
        });
        it("throws Error when provider not provided", function () {
            expect(function () {
                new pact_web_1.PactWeb({ consumer: "someconsumer", provider: "" });
            }).not.to.throw(Error, "You must specify a Provider for this pact.");
        });
    });
    describe("#addInteraction", function () {
        var interaction = {
            state: "i have a list of projects",
            uponReceiving: "a request for projects",
            withRequest: {
                headers: { Accept: "application/json" },
                method: request_1.HTTPMethod.GET,
                path: "/projects",
            },
            willRespondWith: {
                body: {},
                headers: { "Content-Type": "application/json" },
                status: 200,
            },
        };
        describe("when given a provider state", function () {
            it("creates interaction with state", function (done) {
                var pact = Object.create(pact_web_1.PactWeb.prototype);
                pact.opts = fullOpts;
                pact.mockService = {
                    addInteraction: function (int) { return Promise.resolve(int.state); },
                };
                expect(pact.addInteraction(interaction))
                    .to.eventually.have.property("providerState")
                    .notify(done);
            });
        });
        describe("when not given a provider state", function () {
            it("creates interaction with state", function (done) {
                var pact = Object.create(pact_web_1.PactWeb.prototype);
                pact.opts = fullOpts;
                pact.mockService = {
                    addInteraction: function (int) { return Promise.resolve(int.state); },
                };
                var interactionWithNoState = interaction;
                interactionWithNoState.state = undefined;
                expect(pact.addInteraction(interaction))
                    .to.eventually.not.have.property("providerState")
                    .notify(done);
            });
            describe("when given an Interaction as a builder", function () {
                it("creates interaction", function () {
                    var interaction2 = new interaction_1.Interaction()
                        .given("i have a list of projects")
                        .uponReceiving("a request for projects")
                        .withRequest({
                        method: request_1.HTTPMethod.GET,
                        path: "/projects",
                        headers: { Accept: "application/json" },
                    })
                        .willRespondWith({
                        status: 200,
                        headers: { "Content-Type": "application/json" },
                        body: {},
                    });
                    var pact = Object.create(pact_web_1.PactWeb.prototype);
                    pact.opts = fullOpts;
                    pact.mockService = {
                        addInteraction: function (int) {
                            return Promise.resolve(int);
                        },
                    };
                    return expect(pact.addInteraction(interaction2)).to.eventually.have.property("given");
                });
            });
        });
    });
    describe("#verify", function () {
        describe("when pact verification is successful", function () {
            it("should return a successful promise and remove interactions", function (done) {
                var verifyStub = sandbox.stub(mockService_1.MockService.prototype, "verify");
                verifyStub.resolves("verified!");
                var removeInteractionsStub = sandbox.stub(mockService_1.MockService.prototype, "removeInteractions");
                removeInteractionsStub.resolves("removeInteractions");
                var b = Object.create(pact_web_1.PactWeb.prototype);
                b.opts = fullOpts;
                b.mockService = {
                    verify: verifyStub,
                    removeInteractions: removeInteractionsStub,
                };
                var verifyPromise = b.verify();
                expect(verifyPromise).to.eventually.eq("removeInteractions");
                expect(verifyPromise).to.eventually.be.fulfilled.notify(done);
            });
        });
        describe("when pact verification is unsuccessful", function () {
            it("should throw an error", function (done) {
                var verifyStub = sandbox.stub(mockService_1.MockService.prototype, "verify");
                verifyStub.rejects("not verified!");
                var removeInteractionsStub = sandbox.stub(mockService_1.MockService.prototype, "removeInteractions");
                removeInteractionsStub.resolves("removeInteractions");
                var b = Object.create(pact_web_1.PactWeb.prototype);
                b.opts = fullOpts;
                b.mockService = {
                    verify: verifyStub,
                    removeInteractions: removeInteractionsStub,
                };
                var verifyPromise = b.verify();
                expect(verifyPromise)
                    .to.eventually.be.rejectedWith(Error)
                    .notify(done);
                verifyPromise.catch(function (e) {
                    expect(removeInteractionsStub).to.callCount(0);
                });
            });
        });
        describe("when pact verification is successful", function () {
            describe("and an error is thrown in the cleanup", function () {
                it("should throw an error", function (done) {
                    var verifyStub = sandbox.stub(mockService_1.MockService.prototype, "verify");
                    verifyStub.resolves("verified!");
                    var removeInteractionsStub = sandbox.stub(mockService_1.MockService.prototype, "removeInteractions");
                    removeInteractionsStub.throws(new Error("error removing interactions"));
                    var b = Object.create(pact_web_1.PactWeb.prototype);
                    b.opts = fullOpts;
                    b.mockService = {
                        verify: verifyStub,
                        removeInteractions: removeInteractionsStub,
                    };
                    expect(b.verify())
                        .to.eventually.be.rejectedWith(Error)
                        .notify(done);
                });
            });
        });
    });
    describe("#finalize", function () {
        describe("when writing Pact is successful", function () {
            it("should return a successful promise and shut down down the mock server", function (done) {
                var writePactStub = sandbox
                    .stub(mockService_1.MockService.prototype, "writePact")
                    .resolves("pact file written!");
                var p = Object.create(pact_web_1.PactWeb.prototype);
                p.opts = fullOpts;
                p.mockService = {
                    writePact: writePactStub,
                    removeInteractions: sandbox.stub(),
                };
                var writePactPromise = p.finalize();
                expect(writePactPromise).to.eventually.be.fulfilled.notify(done);
            });
        });
        describe("when writing Pact is unsuccessful", function () {
            it("should throw an error", function (done) {
                var writePactStub = sandbox
                    .stub(mockService_1.MockService.prototype, "writePact")
                    .rejects("pact not file written!");
                var p = Object.create(pact_web_1.PactWeb.prototype);
                p.opts = fullOpts;
                p.mockService = {
                    writePact: writePactStub,
                    removeInteractions: sandbox.stub(),
                };
                var writePactPromise = p.finalize();
                expect(writePactPromise)
                    .to.eventually.be.rejectedWith(Error)
                    .notify(done);
            });
        });
    });
    describe("#writePact", function () {
        describe("when writing Pact is successful", function () {
            it("should return a successful promise", function (done) {
                var writePactStub = sandbox
                    .stub(mockService_1.MockService.prototype, "writePact")
                    .resolves("pact file written!");
                var p = Object.create(pact_web_1.PactWeb.prototype);
                p.opts = fullOpts;
                p.mockService = {
                    writePact: writePactStub,
                    removeInteractions: sandbox.stub(),
                };
                var writePactPromise = p.writePact();
                expect(writePactPromise).to.eventually.eq("pact file written!");
                expect(writePactPromise).to.eventually.be.fulfilled.notify(done);
            });
        });
    });
    describe("#removeInteractions", function () {
        describe("when removing interactions is successful", function () {
            it("should return a successful promise", function (done) {
                var removeInteractionsStub = sandbox
                    .stub(mockService_1.MockService.prototype, "removeInteractions")
                    .resolves("interactions removed!");
                var p = Object.create(pact_web_1.PactWeb.prototype);
                p.opts = fullOpts;
                p.mockService = {
                    removeInteractions: removeInteractionsStub,
                };
                var removeInteractionsPromise = p.removeInteractions();
                expect(removeInteractionsPromise).to.eventually.eq("interactions removed!");
                expect(removeInteractionsPromise).to.eventually.be.fulfilled.notify(done);
            });
        });
    });
});
//# sourceMappingURL=pact-web.spec.js.map