"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/* tslint:disable:no-unused-expression object-literal-sort-keys no-empty no-console */
var Promise = require("bluebird");
var chai = require("chai");
var chaiAsPromised = require("chai-as-promised");
var path = require("path");
var superagent = require("superagent");
var request_1 = require("./common/request");
var pact_1 = require("./pact");
chai.use(chaiAsPromised);
var expect = chai.expect;
var eachLike = pact_1.Matchers.eachLike, like = pact_1.Matchers.like, term = pact_1.Matchers.term;
describe("Integration", function () {
    var protocols = ["http", "https"];
    protocols.forEach(function (protocol) {
        describe("Pact on " + protocol + " protocol", function () {
            var MOCK_PORT = Math.floor(Math.random() * 999) + 9000;
            var PROVIDER_URL = protocol + "://localhost:" + MOCK_PORT;
            var provider = new pact_1.Pact({
                consumer: "Matching Service",
                provider: "Animal Profile Service",
                port: MOCK_PORT,
                log: path.resolve(process.cwd(), "logs", "mockserver-integration.log"),
                dir: path.resolve(process.cwd(), "pacts"),
                logLevel: "warn",
                ssl: protocol === "https",
                spec: 2,
            });
            var EXPECTED_BODY = [
                {
                    id: 1,
                    name: "Project 1",
                    due: "2016-02-11T09:46:56.023Z",
                    tasks: [
                        {
                            id: 1,
                            name: "Do the laundry",
                            done: true,
                        },
                        {
                            id: 2,
                            name: "Do the dishes",
                            done: false,
                        },
                        {
                            id: 3,
                            name: "Do the backyard",
                            done: false,
                        },
                        {
                            id: 4,
                            name: "Do nothing",
                            done: false,
                        },
                    ],
                },
            ];
            before(function () { return provider.setup(); });
            // once all tests are run, write pact and remove interactions
            after(function () { return provider.finalize(); });
            context("with a single request", function () {
                // add interactions, as many as needed
                before(function () {
                    return provider.addInteraction({
                        state: "i have a list of projects",
                        uponReceiving: "a request for projects",
                        withRequest: {
                            method: request_1.HTTPMethod.GET,
                            path: "/projects",
                            headers: {
                                Accept: "application/json",
                            },
                        },
                        willRespondWith: {
                            status: 200,
                            headers: {
                                "Content-Type": "application/json",
                            },
                            body: EXPECTED_BODY,
                        },
                    });
                });
                // execute your assertions
                it("returns the correct body", function () {
                    return superagent
                        .get(PROVIDER_URL + "/projects")
                        .set({
                        Accept: "application/json",
                    })
                        .then(function (res) {
                        expect(res.text).to.eql(JSON.stringify(EXPECTED_BODY));
                    });
                });
                // verify with Pact, and reset expectations
                it("successfully verifies", function () { return provider.verify(); });
            });
            context("with a single request with query string parameters", function () {
                // add interactions, as many as needed
                before(function () {
                    return provider.addInteraction({
                        state: "i have a list of projects",
                        uponReceiving: "a request for projects with a filter",
                        withRequest: {
                            method: request_1.HTTPMethod.GET,
                            path: "/projects",
                            query: {
                                from: "today",
                            },
                            headers: {
                                Accept: "application/json",
                            },
                        },
                        willRespondWith: {
                            status: 200,
                            headers: {
                                "Content-Type": "application/json",
                            },
                            body: EXPECTED_BODY,
                        },
                    });
                });
                // execute your assertions
                it("returns the correct body", function () {
                    return superagent
                        .get(PROVIDER_URL + "/projects?from=today")
                        .set({
                        Accept: "application/json",
                    })
                        .then(function (res) {
                        expect(res.text).to.eql(JSON.stringify(EXPECTED_BODY));
                    });
                });
                // verify with Pact, and reset expectations
                it("successfully verifies", function () { return provider.verify(); });
            });
            context("with a single request and eachLike, like, term", function () {
                // add interactions, as many as needed
                before(function () {
                    return provider.addInteraction({
                        state: "i have a list of projects but I dont know how many",
                        uponReceiving: "a request for such projects",
                        withRequest: {
                            method: request_1.HTTPMethod.GET,
                            path: "/projects",
                            headers: {
                                Accept: "application/json",
                            },
                        },
                        willRespondWith: {
                            status: 200,
                            headers: {
                                "Content-Type": term({
                                    generate: "application/json",
                                    matcher: "application/json",
                                }),
                            },
                            body: [
                                {
                                    id: 1,
                                    name: "Project 1",
                                    due: "2016-02-11T09:46:56.023Z",
                                    tasks: eachLike({
                                        id: like(1),
                                        name: like("Do the laundry"),
                                        done: like(true),
                                    }, {
                                        min: 4,
                                    }),
                                },
                            ],
                        },
                    });
                });
                // execute your assertions
                it("returns the correct body", function () {
                    var verificationPromise = superagent
                        .get(PROVIDER_URL + "/projects")
                        .set({ Accept: "application/json" })
                        .then(function (res) { return JSON.parse(res.text)[0]; });
                    return expect(verificationPromise).to.eventually.have.property("tasks");
                });
                // verify with Pact, and reset expectations
                it("successfully verifies", function () { return provider.verify(); });
            });
            context("with two requests", function () {
                before(function () {
                    var interaction1 = provider.addInteraction({
                        state: "i have a list of projects",
                        uponReceiving: "a request for projects",
                        withRequest: {
                            method: request_1.HTTPMethod.GET,
                            path: "/projects",
                            headers: {
                                Accept: "application/json",
                            },
                        },
                        willRespondWith: {
                            status: 200,
                            headers: {
                                "Content-Type": "application/json",
                            },
                            body: EXPECTED_BODY,
                        },
                    });
                    var interaction2 = provider.addInteraction({
                        state: "i have a list of projects",
                        uponReceiving: "a request for a project that does not exist",
                        withRequest: {
                            method: request_1.HTTPMethod.GET,
                            path: "/projects/2",
                            headers: {
                                Accept: "application/json",
                            },
                        },
                        willRespondWith: {
                            status: 404,
                            headers: {
                                "Content-Type": "application/json",
                            },
                        },
                    });
                    return Promise.all([interaction1, interaction2]);
                });
                it("allows two requests", function () {
                    var verificationPromise = superagent
                        .get(PROVIDER_URL + "/projects")
                        .set({
                        Accept: "application/json",
                    })
                        .then(function (res) { return res.text; });
                    var verificationPromise404 = superagent
                        .get(PROVIDER_URL + "/projects/2")
                        .set({
                        Accept: "application/json",
                    });
                    return Promise.all([
                        expect(verificationPromise).to.eventually.equal(JSON.stringify(EXPECTED_BODY)),
                        expect(verificationPromise404).to.eventually.be.rejected,
                    ]);
                });
                // verify with Pact, and reset expectations
                it("successfully verifies", function () { return provider.verify(); });
            });
            context("with an unexpected interaction", function () {
                // add interactions, as many as needed
                before(function () {
                    return provider
                        .addInteraction({
                        state: "i have a list of projects",
                        uponReceiving: "a request for projects",
                        withRequest: {
                            method: request_1.HTTPMethod.GET,
                            path: "/projects",
                            headers: {
                                Accept: "application/json",
                            },
                        },
                        willRespondWith: {
                            status: 200,
                            headers: {
                                "Content-Type": "application/json",
                            },
                            body: EXPECTED_BODY,
                        },
                    })
                        .then(function () { return console.log("Adding interaction worked"); }, function () { return console.warn("Adding interaction failed."); });
                });
                it("fails verification", function () {
                    var verificationPromise = superagent
                        .get(PROVIDER_URL + "/projects")
                        .set({ Accept: "application/json" })
                        .then(function () {
                        return superagent.delete(PROVIDER_URL + "/projects/2").catch(function () { });
                    })
                        .then(function () { return provider.verify(); });
                    return expect(verificationPromise).to.be.rejectedWith("Pact verification failed - expected interactions did not match actual.");
                });
            });
        });
    });
});
//# sourceMappingURL=pact.integration.spec.js.map