"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/* tslint:disable:no-unused-expression object-literal-sort-keys max-classes-per-file no-empty no-console */
var chai = require("chai");
var chaiAsPromised = require("chai-as-promised");
var sinon = require("sinon");
var sinonChai = require("sinon-chai");
var request_1 = require("./common/request");
var interaction_1 = require("./dsl/interaction");
var mockService_1 = require("./dsl/mockService");
var pact_1 = require("./pact");
chai.use(sinonChai);
chai.use(chaiAsPromised);
var expect = chai.expect;
var proxyquire = require("proxyquire").noCallThru();
// Mock out the PactNode interfaces
var PactServer = /** @class */ (function () {
    function PactServer() {
    }
    PactServer.prototype.start = function () { };
    PactServer.prototype.delete = function () { };
    return PactServer;
}());
var PactNodeFactory = /** @class */ (function () {
    function PactNodeFactory() {
    }
    PactNodeFactory.prototype.logLevel = function (opts) { };
    PactNodeFactory.prototype.removeAllServers = function () { };
    PactNodeFactory.prototype.createServer = function (opts) { };
    return PactNodeFactory;
}());
describe("Pact", function () {
    var fullOpts = {
        consumer: "A",
        provider: "B",
        port: 1234,
        host: "127.0.0.1",
        ssl: false,
        logLevel: "info",
        spec: 2,
        cors: false,
        pactfileWriteMode: "overwrite",
    };
    var sandbox = sinon.sandbox.create({
        injectInto: null,
        properties: ["spy", "stub", "mock"],
        useFakeTimers: false,
        useFakeServer: false,
    });
    afterEach(function () {
        sandbox.restore();
    });
    describe("#constructor", function () {
        var Pact;
        beforeEach(function () {
            var imported = proxyquire("./pact", {
                "@pact-foundation/pact-node": sinon.createStubInstance(PactNodeFactory),
            });
            Pact = imported.Pact;
        });
        it("throws Error when consumer not provided", function () {
            expect(function () {
                new Pact({ consumer: "", provider: "provider" });
            }).to.throw(Error, "You must specify a Consumer for this pact.");
        });
        it("throws Error when provider not provided", function () {
            expect(function () {
                new Pact({ consumer: "someconsumer", provider: "" });
            }).to.throw(Error, "You must specify a Provider for this pact.");
        });
    });
    describe("#createOptionsWithDefault", function () {
        var constructorOpts = {
            consumer: "A",
            provider: "B",
        };
        it("should merge options with sensible defaults", function () {
            var opts = pact_1.Pact.createOptionsWithDefaults(constructorOpts);
            expect(opts.consumer).to.eq("A");
            expect(opts.provider).to.eq("B");
            expect(opts.cors).to.eq(false);
            expect(opts.host).to.eq("127.0.0.1");
            expect(opts.logLevel).to.eq("info");
            expect(opts.spec).to.eq(2);
            expect(opts.dir).not.to.be.empty;
            expect(opts.log).not.to.be.empty;
            expect(opts.pactfileWriteMode).to.eq("overwrite");
            expect(opts.ssl).to.eq(false);
            expect(opts.sslcert).to.eq(undefined);
            expect(opts.sslkey).to.eq(undefined);
        });
    });
    describe("#setup", function () {
        var Pact = pact_1.Pact;
        describe("when server is not properly configured", function () {
            describe("and pact-node is unable to start the server", function () {
                it("should return a rejected promise", function () {
                    // TODO: actually test is pact-node is failing on start with a bad config instead of stubbing it
                    var startStub = sandbox
                        .stub(PactServer.prototype, "start")
                        .throws("start");
                    startStub.rejects();
                    var b = Object.create(Pact.prototype);
                    b.opts = fullOpts;
                    b.server = { start: startStub, options: { port: 1234 } };
                    return expect(b.setup()).to.eventually.be.rejected;
                });
            });
        });
        describe("when server is properly configured", function () {
            it("should start the mock server in the background", function () {
                // TODO: actually test is pact-node is starting instead of stubbing it
                var startStub = sandbox.stub(PactServer.prototype, "start");
                startStub.resolves();
                var b = Object.create(Pact.prototype);
                b.opts = fullOpts;
                b.server = { start: startStub, options: { port: 1234 } };
                return expect(b.setup()).to.eventually.be.fulfilled;
            });
        });
        describe("when server is properly configured", function () {
            it("should return the current configuration", function () {
                // TODO: actually test is pact-node is starting instead of stubbing it
                var startStub = sandbox.stub(PactServer.prototype, "start");
                startStub.resolves();
                var b = Object.create(Pact.prototype);
                b.opts = fullOpts;
                b.server = { start: startStub, options: { port: 1234 } };
                return expect(b.setup()).to.eventually.include({
                    consumer: "A",
                    provider: "B",
                    port: 1234,
                    host: "127.0.0.1",
                    ssl: false,
                    logLevel: "info",
                    spec: 2,
                    cors: false,
                    pactfileWriteMode: "overwrite",
                });
            });
        });
    });
    describe("#addInteraction", function () {
        var Pact = pact_1.Pact;
        var interaction = {
            state: "i have a list of projects",
            uponReceiving: "a request for projects",
            withRequest: {
                method: request_1.HTTPMethod.GET,
                path: "/projects",
                headers: { Accept: "application/json" },
            },
            willRespondWith: {
                status: 200,
                headers: { "Content-Type": "application/json" },
                body: {},
            },
        };
        describe("when given a provider state", function () {
            it("creates interaction with state", function () {
                var pact = Object.create(Pact.prototype);
                pact.opts = fullOpts;
                pact.mockService = {
                    addInteraction: function (int) { return Promise.resolve(int.state); },
                };
                return expect(pact.addInteraction(interaction)).to.eventually.have.property("providerState");
            });
        });
        describe("when not given a provider state", function () {
            it("creates interaction with no state", function () {
                var pact = Object.create(Pact.prototype);
                pact.opts = fullOpts;
                pact.mockService = {
                    addInteraction: function (int) { return Promise.resolve(int.state); },
                };
                interaction.state = undefined;
                return expect(pact.addInteraction(interaction)).to.eventually.not.have.property("providerState");
            });
            describe("when given an Interaction as a builder", function () {
                it("creates interaction", function () {
                    var interaction2 = new interaction_1.Interaction()
                        .given("i have a list of projects")
                        .uponReceiving("a request for projects")
                        .withRequest({
                        method: request_1.HTTPMethod.GET,
                        path: "/projects",
                        headers: { Accept: "application/json" },
                    })
                        .willRespondWith({
                        status: 200,
                        headers: { "Content-Type": "application/json" },
                        body: {},
                    });
                    var pact = Object.create(Pact.prototype);
                    pact.opts = fullOpts;
                    pact.mockService = {
                        addInteraction: function (int) {
                            return Promise.resolve(int);
                        },
                    };
                    return expect(pact.addInteraction(interaction2)).to.eventually.have.property("given");
                });
            });
        });
    });
    describe("#verify", function () {
        var Pact = pact_1.Pact;
        describe("when pact verification is successful", function () {
            it("should return a successful promise and remove interactions", function () {
                var verifyStub = sandbox
                    .stub(mockService_1.MockService.prototype, "verify")
                    .resolves("verified!");
                var removeInteractionsStub = sandbox
                    .stub(mockService_1.MockService.prototype, "removeInteractions")
                    .resolves("removeInteractions");
                var b = Object.create(Pact.prototype);
                b.opts = fullOpts;
                b.mockService = {
                    verify: verifyStub,
                    removeInteractions: removeInteractionsStub,
                };
                var verifyPromise = b.verify();
                return Promise.all([
                    expect(verifyPromise).to.eventually.eq("removeInteractions"),
                    expect(verifyPromise).to.eventually.be.fulfilled,
                ]);
            });
        });
        describe("when pact verification is unsuccessful", function () {
            it("should throw an error", function () {
                var verifyStub = sandbox
                    .stub(mockService_1.MockService.prototype, "verify")
                    .rejects("not verified!");
                var removeInteractionsStub = sandbox
                    .stub(mockService_1.MockService.prototype, "removeInteractions")
                    .resolves("removeInteractions");
                var b = Object.create(Pact.prototype);
                b.opts = fullOpts;
                b.mockService = {
                    verify: verifyStub,
                    removeInteractions: removeInteractionsStub,
                };
                var verifyPromise = b.verify();
                return Promise.all([
                    expect(verifyPromise).to.eventually.be.rejectedWith(Error),
                    verifyPromise.catch(function () {
                        return expect(removeInteractionsStub).to.callCount(1);
                    }),
                ]);
            });
        });
        describe("when pact verification is successful", function () {
            describe("and an error is thrown in the cleanup", function () {
                it("should throw an error", function () {
                    var verifyStub = sandbox
                        .stub(mockService_1.MockService.prototype, "verify")
                        .resolves("verified!");
                    var removeInteractionsStub = sandbox.stub(mockService_1.MockService.prototype, "removeInteractions");
                    removeInteractionsStub.throws(new Error("error removing interactions"));
                    var b = Object.create(Pact.prototype);
                    b.opts = fullOpts;
                    b.mockService = {
                        verify: verifyStub,
                        removeInteractions: removeInteractionsStub,
                    };
                    return expect(b.verify()).to.eventually.be.rejectedWith(Error);
                });
            });
        });
    });
    describe("#finalize", function () {
        var Pact = pact_1.Pact;
        describe("when writing Pact is successful", function () {
            it("should return a successful promise and shut down down the mock server", function () {
                var writePactStub = sandbox
                    .stub(mockService_1.MockService.prototype, "writePact")
                    .resolves();
                var p = Object.create(Pact.prototype);
                p.opts = fullOpts;
                p.mockService = {
                    writePact: writePactStub,
                    removeInteractions: sandbox.stub(),
                };
                p.server = {
                    delete: sandbox.stub(PactServer.prototype, "delete").resolves(),
                };
                return expect(p.finalize()).to.eventually.be.fulfilled;
            });
        });
        describe("when writing Pact is unsuccessful", function () {
            it("should throw an error and shut down the server", function () {
                var writePactStub = sandbox
                    .stub(mockService_1.MockService.prototype, "writePact")
                    .rejects();
                var deleteStub = sandbox
                    .stub(PactServer.prototype, "delete")
                    .resolves();
                var p = Object.create(Pact.prototype);
                p.opts = fullOpts;
                p.mockService = {
                    writePact: writePactStub,
                    removeInteractions: sandbox.stub(),
                };
                p.server = { delete: deleteStub };
                return expect(p.finalize()).to.eventually.be.rejected.then(function () {
                    return expect(deleteStub).to.callCount(1);
                });
            });
        });
        describe("when writing pact is successful and shutting down the mock server is unsuccessful", function () {
            it("should throw an error", function () {
                var writePactStub = sandbox
                    .stub(mockService_1.MockService.prototype, "writePact")
                    .resolves();
                var p = Object.create(Pact.prototype);
                p.opts = fullOpts;
                p.mockService = {
                    writePact: writePactStub,
                    removeInteractions: sandbox.stub(),
                };
                p.server = {
                    delete: sandbox.stub(PactServer.prototype, "delete").rejects(),
                };
                return expect(p.finalize).to.throw(Error);
            });
        });
    });
    describe("#writePact", function () {
        var Pact = pact_1.Pact;
        describe("when writing Pact is successful", function () {
            it("should return a successful promise", function () {
                var writePactStub = sandbox
                    .stub(mockService_1.MockService.prototype, "writePact")
                    .resolves("pact file written!");
                var p = Object.create(Pact.prototype);
                p.opts = fullOpts;
                p.mockService = {
                    writePact: writePactStub,
                    removeInteractions: sandbox.stub(),
                };
                var writePactPromise = p.writePact();
                return Promise.all([
                    expect(writePactPromise).to.eventually.eq("pact file written!"),
                    expect(writePactPromise).to.eventually.be.fulfilled,
                ]);
            });
        });
    });
    describe("#removeInteractions", function () {
        var Pact = pact_1.Pact;
        describe("when removing interactions is successful", function () {
            it("should return a successful promise", function () {
                var removeInteractionsStub = sandbox
                    .stub(mockService_1.MockService.prototype, "removeInteractions")
                    .resolves("interactions removed!");
                var p = Object.create(Pact.prototype);
                p.opts = fullOpts;
                p.mockService = {
                    removeInteractions: removeInteractionsStub,
                };
                var removeInteractionsPromise = p.removeInteractions();
                return Promise.all([
                    expect(removeInteractionsPromise).to.eventually.eq("interactions removed!"),
                    expect(removeInteractionsPromise).to.eventually.be.fulfilled,
                ]);
            });
        });
    });
});
//# sourceMappingURL=pact.spec.js.map